# NTray - A tray notification tool for the Nagios Monitoring System
# Copyright (C) 2002 Robert M. Wagner (rob@robandsara.com)
# Licensed under the GPL (see included LICENSE file for full license)
# 
# Nagios is a registered trademark of Ethan Galstad. (nagios@nagios.org)
# You can visit the Nagios home page at http://www.nagios.org
#
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

my $version = "v0.92"; #Just for reference :)

use XML::Simple;
use Win32::GUI;
use Win32::Shell;
use Crypt::SSLeay;
use LWP::UserAgent;
use URI::https; #perl2exe
use LWP::Protocol::https; #perl2exe
use Net::SSL; #perl2exe
use Win32::Sound;
use Strict;

#Needed some globals
my $previous_color;
my $xml;
my $config;


#START EXECUTION
main();


sub parse_command_line {
   my @commandline = @ARGV;
   my $override_configfile = "None";
   my $override_refresh = 0;


   while (scalar(@commandline) > 0) {
      my $command = shift(@commandline);
  
      if ($command =~ /--help/i) {
         Win32::GUI::MessageBox(0, "NTray $version Usage:\nNTray.exe \[-c d:\\path\\config.xml\] \[-w seconds_to_refresh\]\n\nExample:\nNTray.exe -c c:\\etc\\ntrayconfig.xml -w 60\n", "NTray Help");
         die ("NTray $version Usage:\nNTray.exe \[-c d:\\path\\config.xml\] \[-r seconds_to_refresh\]\n\nExample:\nNTray.exe -c c:\\etc\\ntrayconfig.xml -r 60\n");
      }
      
      if ($command =~ /-c/) {
         my $override_configfile = shift(@commandline);
         print "Overriding config file default. Using $override_configfile\n";
      }

      if ($command =~ /-r/) {
         my $override_refresh = shift(@commandline);
         print "Overriding refresh time from config file. Using $override_refresh\n";
      }
   }
   return ($override_configfile,$override_refresh);
}

sub check_configfile {
   my ($override_configfile) = @_;
   my $configfile = "./config.xml";

   if ($override_configfile ne "None") {
      $configfile = $override_configfile;
   }

   if (! -e "$configfile") {
      #Prompt - Config file not found - Do you want to configure?
      return (0,$configfile);
   }
   return (1,$configfile);
}

sub check_icons {
   #Verify that all required icons exist and can be used
   if ((-e "green.ico") && (-e "yellow.ico") && (-e "red.ico") && (-e "unknown.ico")) {
      return 1;
   } else {
      return 0;
   }
}


sub main {
   #Initialization
   my ($override_configfile,$override_refresh) = parse_command_line();
   if (! check_icons()) {
      Win32::GUI::MessageBox(0, "Application Error: Some or all icons are missing!\nMake sure red.ico, green.ico, yellow.ico, \nand unknown.ico exist in the current directory.", "NTray Error");
      die ("Application Error: Some or all icons are missing!\nMake sure red.ico, green.ico, yellow.ico, \nand unknown.ico exist in the current directory.");
   }
   
   my ($check_configfile,$configfile) = check_configfile($override_configfile);
   if (! $check_configfile) {
      #Replace error here with dialogbox to prompt if user wants to configure NTray or not.
      Win32::GUI::MessageBox(0, "Application Error: Configuration file not found! ($configfile)", "NTray Error");
      die ("Application Error: Configuration file not found! ($configfile)");
   }

   #Read in the config file, since we've already tested for it's existence.
   $xml = XML::Simple->new();
   $config = $xml->XMLin("$configfile", keeproot => 1);   

   #'Decrypt' the password fields in the config file
   $config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD} = join("",   map { chr } (split /\\/,$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD} ));
   $config->{CONFIG}->{PROXY}->{PROXY_PASSWORD} = join("",   map { chr } (split /\\/,$config->{CONFIG}->{PROXY}->{PROXY_PASSWORD}));


   my $refresh = 0;
   if ($override_refresh > 0) {
      print "$override_refresh\n";
      $refresh = $override_refresh * 1000;
   } else {
      $refresh = $config->{CONFIG}->{BASIC}->{REFRESH_TIMER} * 1000;
   }

   #Create master window, icon and timer objects
   create_winMaster();
   create_tmrMaster($refresh);
   create_icons();

   #Create configuration window
   create_winConfig();

   #Get initial status from webserver (other object creates are dependent on this)
   my ($status_color,$icon_text,$status_text) = getStatus();
   $previous_color = $status_color;
   
   #Create Popup Window
   create_winPopup($status_color,$status_text);
   
   #Create Initial Icon
   create_icoTrayicon($status_color,$icon_text);


   #And finally.. start the dialog loop
   Win32::GUI::Dialog();

   #On termination, write config
   write_configfile("$configfile");
}

sub create_winMaster {
   $winMaster = Win32::GUI::Window->new(-name => 'winMaster', -text => 'NTray',
                                        -width => 10, -height => 10,
                                        -left => 10, -top => 10);
}

sub create_tmrMaster {
   my ($refresh) = @_; 	
   $tmrMaster = $winMaster->AddTimer(tmrMaster,$refresh);
}

sub create_icons {
   $green = new Win32::GUI::Icon("green.ico");
   $yellow = new Win32::GUI::Icon("yellow.ico");
   $red = new Win32::GUI::Icon("red.ico");
   $unknown = new Win32::GUI::Icon("unknown.ico");
}

sub create_winPopup {
   my ($status_color,$status_text) = @_;
   $winPopup = Win32::GUI::Window->new(-name => 'winPopup', -text => 'NTray Popup Notification',
                                    -width => 440, -height => 240,
                                    -left  => $config->{CONFIG}->{POPUPS}->{POPUP_XPOS}, 
                                    -top   => $config->{CONFIG}->{POPUPS}->{POPUP_YPOS},
                                    -icon  => $$status_color);

   $gbPopup = Win32::GUI::Groupbox->new($winPopup,
                                        -text    => "Nagios Status: $status_color",
                                        -name    => "gbPopStatus",
                                        -left    => 4,
                                        -top     => 4,
                                        -width   => 420,
                                        -height  => 200,
                                        );

   $lblPopup = Win32::GUI::Label->new($winPopup,
                                      -text    => "$status_text\n\n",
                                      -name    => "lblPopStatus",
                                      -left    => 14,
                                      -top     => 20,
                                      -width   => 400,
                                      -height  => 180,
                                      );
}


sub create_winConfig {
   $winConfig = Win32::GUI::Window->new(-name => 'winConfig',
                                        -text => 'NTray Configuration',
                                        -width => 500, -height => 480,
                                        -left => $config->{CONFIG}->{CONFIG_WIN}->{CONFIG_WIN_XPOS}, 
                                        -top => $config->{CONFIG}->{CONFIG_WIN}->{CONFIG_WIN_YPOS},
                                        );
                                        
   $gbConfig = Win32::GUI::Groupbox->new($winConfig,
                                         -text    => "NTray Configuration",
                                         -name    => "gbConfig",
                                         -left    => 4,
                                         -top     => 4,
                                         -width   => 480,
                                         -height  => 450,
                                         );

   
   $tfCfgcgiurl = Win32::GUI::Textfield->new($winConfig,
                                             -text   => "$config->{CONFIG}->{BASIC}->{CGI_URL}",
                                             -name   => "tfCfgcgiurl",
                                             -left   => 20,
                                             -top    => 50,
                                             -width  => 330,
                                             -height => 20,
                                             -prompt => ["Nagios CGI URL",110],
                                             );

   $tfCfgnagiosurl = Win32::GUI::Textfield->new($winConfig,
                                                -text   => "$config->{CONFIG}->{BASIC}->{NAGIOS_URL}",
                                                -name	=> "tfCfgnagiosURL",
                                                -left	=> 20,
                                                -top	=> 25,
                                                -width	=> 330,
                                                -height	=> 20,
                                                -prompt => ["Nagios URL",110],
                                                );

   $tfCfgrefresh = Win32::GUI::Textfield->new($winConfig,
                                              -text     => "$config->{CONFIG}->{BASIC}->{REFRESH_TIMER}",
                                              -name     => "tfCfgrefresh",
                                              -left     => 20,
                                              -top      => 75,
                                              -width    => 50,
                                              -height   => 20,
                                              -prompt   => ["Time to Refresh(sec)",110],
                                              );


   $cbCfgsound = Win32::GUI::Checkbox->new($winConfig,
                                           -text    => "Enable Sound",
                                           -name    => "cbCfgsound",
                                           -top     => 100,
                                           -left    => 20,
                                           -width   => 120,
                                           -height  => 20,
                                           );

   $cbCfgpopup = Win32::GUI::Checkbox->new($winConfig,
                                           -text    => "Enable Popup Alerts",
                                           -name    => "cbCfgpopup",
                                           -top	    => 120,
                                           -left    => 20,
                                           -width   => 200,
                                           -height  => 20,
                                           );



   $cbCfgauth = Win32::GUI::Checkbox->new($winConfig,
                                          -text   => "Enable Authentication",
                                          -name   => "cbCfgauth",
                                          -top    => 160,
                                          -left   => 20,
                                          -width  => 130,
                                          -height => 20,
                                          );



   $tfCfgauthname = Win32::GUI::Textfield->new($winConfig,
                                               -text   => "$config->{CONFIG}->{AUTHENTICATION}->{AUTH_USERNAME}",
                                               -name   => "tfCfgauthname",
                                               -left   => 20,
                                               -top    => 185,
                                               -width  => 100,
                                               -height => 20,
                                               -prompt => ["Username",60],
                                               );


   $tfCfgauthpass = Win32::GUI::Textfield->new($winConfig,
                                               -text     => "$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD}",
                                               -name     => "tfCfgauthpass",
                                               -password => 1,
                                               -left     => 20,
                                               -top      => 210,
                                               -width    => 100,
                                               -height   => 20,
                                               -prompt   => ["Password",60],
                                               );




   $cbCfgproxy = Win32::GUI::Checkbox->new($winConfig,
                                           -text   => "Use HTTP Proxy",
                                           -name   => "cbCfgproxy",
                                           -top    => 235,
                                           -left   => 20,
                                           -width  => 130,
                                           -height => 20,
                                           );
                                           

   $tfCfgproxyserver = Win32::GUI::Textfield->new($winConfig,
                                                  -text   => "$config->{CONFIG}->{PROXY}->{PROXY_SERVER}",
                                                  -name   => "tfCfgproxyserver",
                                                  -left   => 20,
                                                  -top    => 260,
                                                  -width  => 300,
                                                  -height => 20,
                                                  -prompt => ["Proxy Server Address",120]
                                                  );


   $tfCfgproxyport = Win32::GUI::Textfield->new($winConfig,
                                                -text   => "$config->{CONFIG}->{PROXY}->{PROXY_PORT}",
                                                -name   => "tfCfgproxyport",
                                                -left   => 20,
                                                -top    => 285,
                                                -width  => 100,
                                                -height => 20,
                                                -prompt => ["Proxy Server Port",120],
                                                );


   $cbCfgproxyauth = Win32::GUI::Checkbox->new($winConfig,
                                               -text   => "Use Proxy Authentication",
                                               -name   => "cbCfgproxyauth",
                                               -top    => 310,
                                               -left   => 40,
                                               -width  => 150,
                                               -height => 20,
                                           );


   $tfCfgproxyuser = Win32::GUI::Textfield->new($winConfig,
                                                -text   => "$config->{CONFIG}->{PROXY}->{PROXY_USERNAME}",
                                                -name   => "tfCfgproxyuser",
                                                -left   => 20,
                                                -top    => 335,
                                                -width  => 300,
                                                -height => 20,
                                                -prompt => ["Proxy Username",120],
                                                );


   $tfCfgproxypass = Win32::GUI::Textfield->new($winConfig,
                                                -text     => "$config->{CONFIG}->{PROXY}->{PROXY_PASSWORD}",
                                                -name     => "tfCfgproxypass",
                                                -password => 1,
                                                -left     => 20,
                                                -top      => 360,
                                                -width    => 100,
                                                -height   => 20,
                                                -prompt   => ["Proxy Password",120],
                                                );

   $btnCfgapply = Win32::GUI::Button->new($winConfig,
                                          -text   => "Apply",
                                          -name   => "btnCfgapply",
                                          -left   => 180,
                                          -top    => 400,
                                          -width  => 80,
                                          -height => 40,
                                          );


   $btnCfgcancel = Win32::GUI::Button->new($winConfig,
                                          -text   => "Cancel",
                                          -name   => "btnCfgcancel",
                                          -left   => 270,
                                          -top    => 400,
                                          -width  => 80,
                                          -height => 40,
                                          );
}

sub write_configfile {
   my ($configfile) = @_;

   #'Encrypt' the password fields in the config file
   $config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD} = join("\\",   map { ord } (split //,$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD} ));
   $config->{CONFIG}->{PROXY}->{PROXY_PASSWORD} = join("\\",   map { ord } (split //,$config->{CONFIG}->{PROXY}->{PROXY_PASSWORD}));

   open (OUTFILE,">$configfile");
   print OUTFILE $xml->XMLout($config, keeproot => 1, noattr => 1);
   close OUTFILE;
}

sub create_icoTrayicon {
   my ($status_color,$status_text) = @_;
   $icoTrayicon = Win32::GUI::NotifyIcon->new($winMaster,
                                           -name => "icoTrayicon",
                                           -tip  => "   Status: $status_text",
                                           -icon => $$status_color,
                                           -id   => $$ ); #Set the ID equal to the PID of this process
   print "Icon Create:\nColor is $status_color\nID is $$\n";
   1,
}

sub change_icon {
   my ($status_color,$status_text) = @_;

   Win32::GUI::NotifyIcon::Modify($winMaster, 
                                  -id => $$, 
                                  -icon => $$status_color, 
                                  -tip => "   Status: $status_text" );

   1; #Don't end the dialog() loop
}

sub tmrMaster_Timer {
   #This is executed at $refresh seconds.
   my ($status_color,$icon_text,$status_text) = getStatus();

   change_icon($status_color,$icon_text); #Do this regardless, just in case status text has changed.

   if ($status_color ne $previous_color) {
      if ($config->{CONFIG}->{SOUND}->{ENABLE_SOUND} =~ /true/i) {
         play_sound($status_color);
      }
      if ($config->{CONFIG}->{POPUPS}->{ENABLE_POPUPS} =~ /true/i) {
         show_popup($status_color,$status_text);
      }
   }
   $previous_color = $status_color;
}

sub play_sound {
   my ($status_color) = @_;
   if (-e "$status_color.wav") {
      Win32::Sound::Play("$status_color.wav", SND_ASYNC);
      #Win32::Sound::Stop();
   }
}

sub show_popup {
   my ($status_color,$status_text) = @_;

   $winPopup->Hide();

   $winPopup->ChangeIcon($$status_color);

   $gbPopup->Text("Nagios Status: $color");
   $lblPopup->Text("\n$status_text");

   $winPopup->Show();
}


sub getStatus {
   #We'll replace this sub in it's entirety later

   #Get the status by doing a quick check of the status.cgi page
   my $color = "unknown";

   my $ua = LWP::UserAgent->new;

   my $url = $config->{CONFIG}->{BASIC}->{CGI_URL} . "/status.cgi?hostgroup=all&style=summary";
   my $req = HTTP::Request->new(GET => $url);

   if ($config->{CONFIG}->{PROXY}->{PROXY_REQUIRED} =~ /true/i) {
      my $proxyaddress = "http://$config->{CONFIG}->{PROXY}->{PROXY_SERVER}:$config->{CONFIG}->{PROXY}->{PROXY_PORT}";
      $ua->proxy('http' => $proxyaddress);
      if ($config->{CONFIG}->{PROXY}->{AUTH_REQUIRED} =~ /true/i) {
         $req->proxy_authorization_basic($config->{CONFIG}->{PROXY}->{PROXY_USERNAME},$config->{CONFIG}->{PROXY}->{PROXY_PASSWORD} );
      }
   }

   if ($config->{CONFIG}->{AUTHENTICATION}->{AUTH_REQUIRED} =~ /true/i) {
      $req->authorization_basic($config->{CONFIG}->{AUTHENTICATION}->{AUTH_USERNAME},$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD});
   }

   my $content = $ua->request($req)->as_string;

   if ($content =~ /last updated/i) {
      $color = "green";
   }

   if ($content =~ /WARNING/) {
      $color = "yellow";
   }

   if ($content =~ /CRITICAL/) {
      $color = "red";
   }

   my $icon_text = "No extended status available at this time.";
   my $status_text = "Checking Nagios Instance:\n$config->{CONFIG}->{BASIC}->{CGI_URL}/status.cgi\n\nNo extended status available at this time.\n(This feature is not yet supported by Nagios or NTray)\n";

   return ($color,$icon_text,$status_text);
}

sub icoTrayicon_Click {
   #Single (left) Click on the Tray Icon (opens Nagios in your default Browser)
   Win32::Shell::Execute("open","$config->{CONFIG}->{BASIC}->{NAGIOS_URL}",undef,undef, "SW_SHOWNORMAL");
   1;
}

sub icoTrayicon_RightClick {
   #Right Click on the Tray Icon
   my ($x,$y) = Win32::GUI::GetCursorPos();

   my $icoMenu = new Win32::GUI::Menu(
      "tray"       => "tray",
      " > &Nagios" => "Nagios",
      " > -"	   => 0,
      " > &Status" => "Status",
      " > &Config" => "Config",
      " > &About"  => "About",
      " > E&xit"   => "Exit",
      );

   $winMaster->TrackPopupMenu($icoMenu->{tray}, $x, $y);

1; #Don't end the dialog loop
}

sub Status_Click {
   my ($status_color,$icon_text,$status_text) = getStatus();

   show_popup($status_color,$status_text);
   1;
}

sub Config_Click {
   show_config();
   
   1;
}

sub About_Click {
	#Click on the 'About' Menu Option
	Win32::GUI::MessageBox(0, "Ntray $version\n\n(c)2002 by Rob Wagner (rob\@robandsara.com)\n\nLicensed Under the GPL\nNtray PID = $$\n", "About NTray");
	1;
}

sub Exit_Click {
   -1;
}


sub winPopup_Terminate {
   #Store Window location for later use
   $config->{CONFIG}->{POPUPS}->{POPUP_XPOS} = $winPopup->Left();
   $config->{CONFIG}->{POPUPS}->{POPUP_YPOS} = $winPopup->Top();

   $winPopup->Hide();
   0;   
}

sub winConfig_Terminate {
   #Store Window location for later use
   $config->{CONFIG}->{CONFIG_WIN}->{CONFIG_WIN_XPOS} = $winConfig->Left();
   $config->{CONFIG}->{CONFIG_WIN}->{CONFIG_WIN_YPOS} = $winConfig->Top();

   $winConfig->Hide();
   0;   
}

sub btnCfgcancel_Click {
   #Same as winConfig_Terminate!
   winConfig_Terminate();
}

sub btnCfgapply_Click {
   #Store our configuration information!!
   $config->{CONFIG}->{BASIC}->{CGI_URL} = $tfCfgcgiurl->Text();
   $config->{CONFIG}->{BASIC}->{NAGIOS_URL} = $tfCfgnagiosurl->Text();
   $config->{CONFIG}->{BASIC}->{REFRESH_TIMER} = $tfCfgrefresh->Text();
   $config->{CONFIG}->{AUTHENTICATION}->{AUTH_USERNAME} = $tfCfgauthname->Text();
   $config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD} = $tfCfgauthpass->Text();
   $config->{CONFIG}->{PROXY}->{PROXY_SERVER} = $tfCfgproxyserver->Text();
   $config->{CONFIG}->{PROXY}->{PROXY_PORT} = $tfCfgproxyport->Text();
   $config->{CONFIG}->{PROXY}->{PROXY_USERNAME} = $tfCfgproxyuser->Text();
   $config->{CONFIG}->{PROXY}->{PROXY_PASSWORD} = $tfCfgproxypass->Text();

   if ($cbCfgsound->Checked()) {
      $config->{CONFIG}->{SOUND}->{ENABLE_SOUND} = "TRUE";
   } else {
      $config->{CONFIG}->{SOUND}->{ENABLE_SOUND} = "FALSE";
   }

   if ($cbCfgpopup->Checked()) {
      $config->{CONFIG}->{POPUPS}->{ENABLE_POPUPS} = "TRUE";
   } else {
      $config->{CONFIG}->{POPUPS}->{ENABLE_POPUPS} = "FALSE";
   }

   if ($cbCfgauth->Checked()) {
      $config->{CONFIG}->{AUTHENTICATION}->{AUTH_REQUIRED} = "TRUE";
   } else {
      $config->{CONFIG}->{AUTHENTICATION}->{AUTH_REQUIRED} = "FALSE";
   }

   if ($cbCfgproxyauth->Checked()) {
      $config->{CONFIG}->{PROXY}->{AUTH_REQUIRED} = "TRUE";
   } else {
      $config->{CONFIG}->{PROXY}->{AUTH_REQUIRED} = "FALSE";
   }

   if ($cbCfgproxy->Checked()) {
      $config->{CONFIG}->{PROXY}->{PROXY_REQUIRED} = "TRUE";
   } else {
      $config->{CONFIG}->{PROXY}->{PROXY_REQUIRED} = "FALSE";
   }

   $config->{CONFIG}->{CONFIG_WIN}->{CONFIG_WIN_XPOS} = $winConfig->Left();
   $config->{CONFIG}->{CONFIG_WIN}->{CONFIG_WIN_YPOS} = $winConfig->Top();

   $winConfig->Hide();
   0;   


}


sub show_config {
   #Show the configuration window.. repopulate values
   $tfCfgcgiurl->Text("$config->{CONFIG}->{BASIC}->{CGI_URL}");
   $tfCfgnagiosurl->Text("$config->{CONFIG}->{BASIC}->{NAGIOS_URL}");
   $tfCfgrefresh->Text("$config->{CONFIG}->{BASIC}->{REFRESH_TIMER}");

   $tmrMaster->Interval($config->{CONFIG}->{BASIC}->{REFRESH_TIMER} * 1000);

   $tfCfgauthname->Text("$config->{CONFIG}->{AUTHENTICATION}->{AUTH_USERNAME}");
   $tfCfgauthpass->Text("$config->{CONFIG}->{AUTHENTICATION}->{AUTH_PASSWORD}");
   $tfCfgproxyserver->Text("$config->{CONFIG}->{PROXY}->{PROXY_SERVER}");
   $tfCfgproxyport->Text("$config->{CONFIG}->{PROXY}->{PROXY_PORT}");
   $tfCfgproxyuser->Text("$config->{CONFIG}->{PROXY}->{PROXY_USERNAME}");
   $tfCfgproxypass->Text("$config->{CONFIG}->{PROXY}->{PROXY_PASSWORD}");

   if ($config->{CONFIG}->{SOUND}->{ENABLE_SOUND} =~ /true/i) {
      $cbCfgsound->Checked(1);
   } else {
      $cbCfgsound->Checked(0);   
   }

   if ($config->{CONFIG}->{POPUPS}->{ENABLE_POPUPS} =~ /true/i) {
      $cbCfgpopup->Checked(1);
   } else {
      $cbCfgpopup->Checked(0);   
   }

   if ($config->{CONFIG}->{AUTHENTICATION}->{AUTH_REQUIRED} =~ /true/i) {
      $cbCfgauth->Checked(1);
      $tfCfgauthname->ReadOnly(0);
      $tfCfgauthpass->ReadOnly(0);
   } else {
      $cbCfgauth->Checked(0);   
      $tfCfgauthname->ReadOnly(1);
      $tfCfgauthpass->ReadOnly(1);
   }

   if ($config->{CONFIG}->{PROXY}->{AUTH_REQUIRED} =~ /true/i) {
      $cbCfgproxyauth->Checked(1);
      $tfCfgproxyuser->ReadOnly(0);
      $tfCfgproxypass->ReadOnly(0);
   } else {
      $cbCfgproxyauth->Checked(0);   
      $tfCfgproxyuser->ReadOnly(1);
      $tfCfgproxypass->ReadOnly(1);
   }


   if ($config->{CONFIG}->{PROXY}->{PROXY_REQUIRED} =~ /true/i) {
      $cbCfgproxy->Checked(1);
      $tfCfgproxyserver->ReadOnly(0);
      $tfCfgproxyport->ReadOnly(0);
      $tfCfgproxyuser->ReadOnly(0);
      $tfCfgproxypass->ReadOnly(0);
   } else {
      $cbCfgproxy->Checked(0);   
      $tfCfgproxyserver->ReadOnly(1);
      $tfCfgproxyport->ReadOnly(1);
      $tfCfgproxyuser->ReadOnly(1);
      $tfCfgproxypass->ReadOnly(1);
   }

   $winConfig->Show();
   1;
}

sub cbCfgauth_Click {
   if ($cbCfgauth->Checked()) {
      $tfCfgauthname->ReadOnly(0);
      $tfCfgauthpass->ReadOnly(0);
   } else {
      $tfCfgauthname->ReadOnly(1);
      $tfCfgauthpass->ReadOnly(1);
   }
}

sub cbCfgproxyauth_Click {
   if ($cbCfgproxyauth->Checked()) {
      if ($cbCfgproxy->Checked()) {
         $tfCfgproxyuser->ReadOnly(0);
         $tfCfgproxypass->ReadOnly(0);
      }
   } else {
      $tfCfgproxyuser->ReadOnly(1);
      $tfCfgproxypass->ReadOnly(1);
   }
}

sub cbCfgproxy_Click {
   if ($cbCfgproxy->Checked()) {
      $tfCfgproxyserver->ReadOnly(0);
      $tfCfgproxyport->ReadOnly(0);
      if ($cbCfgproxyauth->Checked()) {
         $tfCfgproxyuser->ReadOnly(0);
         $tfCfgproxypass->ReadOnly(0);
      }
   } else {
      $tfCfgproxyserver->ReadOnly(1);
      $tfCfgproxyport->ReadOnly(1);
      $tfCfgproxyuser->ReadOnly(1);
      $tfCfgproxypass->ReadOnly(1);
   }
}

